/**
 * 
 */
package org.springblade.analytics.sensors;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;

import com.sensorsdata.analytics.javasdk.ISensorsAnalytics;
import com.sensorsdata.analytics.javasdk.bean.EventRecord;

import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springblade.visual.enums.VisualTypeEnum;

/**
 * @author wookvn
 *
 */
@Slf4j
public class AnalyticsAgent {

	/**
	 * 数据源埋点
	 * @param ds
	 * @param ex
	 * @param sql     - 执行的SQL
	 * @param costMis - 消耗的毫秒数
	 */
	public static void trackDatasource(ReportDatasource ds, Exception ex, String sql, long costMis) {
		String eventName = "useDatasource";
		try {
			ISensorsAnalytics sensorsAnalytics = null;
			try {
				sensorsAnalytics = SpringUtil.getBean(ISensorsAnalytics.class);
			} catch (Exception e) {
			}
			if (sensorsAnalytics == null) {
				log.debug("没有启动神策埋点..");
				return;
			}
			//当ID有值时，ds是包含完整数据的
			if (ds.getId() == null && StrUtil.isBlank(ds.getPoolName())) {
				log.warn("数据源埋点|关键参数为空");
				return;
			}
			//RPC 调用时，只有poolName值
			if (ds.getId() == null && StrUtil.isNotBlank(ds.getPoolName())) {
				//获取数据源service
				String poolName = ds.getPoolName();
				IReportDatasourceService datasourceService = SpringUtil.getBean(IReportDatasourceService.class);
				ds = datasourceService.getCacheByPoolName(poolName);
				if (ds == null) {
					log.warn("数据源埋点|未找到数据源|poolName={}", poolName);
					return;
				}
			}

			Map<String, Object> props = new HashMap<>();
			BladeUser bladeUser = AuthUtil.getUser();
			// -------- 数据源信息 -----------
			props.put("attr1", ds.getId().toString()); //dsId
			props.put("attr2", ds.getDsDb()); //数据库名称
			props.put("attr3", ds.getPoolName()); //poolName
			props.put("attr4", ds.getDsTypeSyl().name()); //dsTypeName
			props.put("attr5", ds.getDsIp()); //dsIp
			props.put("attr6", StrUtil.toStringOrNull(ds.getDsPort())); //dsPort
			props.put("attr7", ds.getDsUser()); //dsUser
			props.put("attr8", ds.getResponsiblePersonName()); //负责人姓名

			// -------- 登录用户信息 ----------
			props.put("attr9", ds.getTenantId()); //tenantId
			props.put("attr10", bladeUser != null ? bladeUser.getNickName() : ""); //realName
			props.put("tenant_id", ds.getTenantId());
			props.put("platform_type", "BE");

			// -------- 数据源指标数据信息 ---------
			props.put("attr11", ex != null && StrUtil.containsIgnoreCase(ex.getMessage(), "connect") ? "1" : "0"); //connect_fail_status
			props.put("attr12", ex == null ? "1" : "0"); //succ_status
			props.put("attr13", ex != null ? "1" : "0"); //fail_status

			//调用失败的描述信息 - 最多存储200个字符
			props.put("attr14", ex != null ? StrUtil.subSufByLength(ex.getMessage(), 200) : "");

			//补充负责人userId属性
			props.put("attr15", ds.getResponsiblePersonId() != null ? ds.getResponsiblePersonId().toString() : "");
			props.put("attr16", sql.replace("\"","'"));
			props.put("attr17", costMis);
			props.put("attr18", ds.getDbSchema());
			props.put("attr19", ds.getDsName());
			//补充租户名称
			props.put("attr20", bladeUser != null ? bladeUser.getDetail().get("enterprise") : "");

			String distinctId = bladeUser != null ? bladeUser.getUserId().toString() : UUID.randomUUID().toString();
			EventRecord.Builder builder = EventRecord.builder().setEventName(eventName)
					.setDistinctId(distinctId).isLoginId(bladeUser != null).addProperties(props);
			try {
				sensorsAnalytics.track(builder.build());
				log.debug("数据源埋点成功.....");
			} catch (Exception e) {
				log.error("数据源埋点失败|" + e.getMessage(), e);
			}
		}catch (Exception ext) {
			log.error(ext.getMessage(), ext);
		}
	}


	/**
	 * 模版的埋点
	 * @param categoryName
	 */
	public static void trackVisualTemplate(Long visualDataId,Integer templateSource, VisualDataTemplate visualDataTemplate,String categoryName) {
		String eventName = "visualPreview";  //事件名称
		try {
			ISensorsAnalytics sensorsAnalytics = null;
			try {
				sensorsAnalytics = SpringUtil.getBean(ISensorsAnalytics.class);
			} catch (Exception e) {
			}
			if (sensorsAnalytics == null) {
				log.debug("没有启动神策埋点..");
				return;
			}
			Map<String, Object> props = new HashMap<>();
			BladeUser bladeUser = AuthUtil.getUser();
			props.put("tenant_id", bladeUser.getTenantId());  //租户ID
			props.put("platform_type", "BE"); //平台类型，BE-后端【默认值，不用修改】

			// -------- 埋点具体信息 -----------
			props.put("attr1", "模板"); //类型
			props.put("attr2", visualDataTemplate.getName()); //名称
			props.put("attr3", bladeUser != null ? bladeUser.getUserId().toString() : "" ); //用户id
			props.put("attr4", bladeUser != null ? bladeUser.getNickName() : ""); //用户昵称
			props.put("attr5",categoryName); //分类名
			props.put("attr6",visualDataTemplate.getTenantId()); //租户ID
			props.put("attr7","pc"); //设备类型pc h5
			props.put("attr8",visualDataTemplate.getId().toString()); //模板id
			props.put("attr9","/newbi/template/preview/"+visualDataTemplate.getId()); //url
			props.put("attr10",visualDataTemplate.getCreateUserName()); //创建人姓名
			props.put("attr11",bladeUser.getDetail().get("enterprise")); //租户名称
			props.put("attr12",visualDataId.toString()); //大屏id
			props.put("attr13",templateSource.toString()); //模板来源

			String distinctId = bladeUser != null ? bladeUser.getUserId().toString() : UUID.randomUUID().toString();
			EventRecord.Builder builder = EventRecord.builder().setEventName(eventName)
					.setDistinctId(distinctId).isLoginId(bladeUser != null).addProperties(props);
			try {
				sensorsAnalytics.track(builder.build());
				log.debug("大屏模版埋点成功.....");
			} catch (Exception e) {
				log.error("大屏模版埋点失败|" + e.getMessage(), e);
			}
		}catch (Exception ext) {
			log.error(ext.getMessage(), ext);
		}
	}
}
