/**
 * 
 */
package org.springblade.config;

import java.io.IOException;
import java.util.Arrays;

import org.springblade.analytics.properties.AnalyticsProperties;
import org.springblade.datasource.support.BladexDataSourceProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.alibaba.fastjson.JSON;
import com.baomidou.dynamic.datasource.creator.DataSourceProperty;
import com.baomidou.dynamic.datasource.creator.DefaultDataSourceCreator;
import com.baomidou.dynamic.datasource.creator.druid.DruidDataSourceCreator;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;
import com.sensorsdata.analytics.javasdk.ISensorsAnalytics;
import com.sensorsdata.analytics.javasdk.SensorsAnalytics;
import com.sensorsdata.analytics.javasdk.consumer.FastBatchConsumer;
import com.sensorsdata.analytics.javasdk.exceptions.InvalidArgumentException;

import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Configuration
public class GemtryBiConfig {
	
	@Autowired
	private AnalyticsProperties analyticsProperties;

	/**
	 * 初始化动态数据源
	 * 
	 * @return
	 */
/*	@Bean
	public BladexDataSourceProvider bdsProvider(DefaultDataSourceCreator defaultDataSourceCreator,
			DynamicDataSourceProperties dynamicDataSourceProperties) {
		DataSourceProperty master = dynamicDataSourceProperties.getDatasource()
				.get(dynamicDataSourceProperties.getPrimary());
		return new BladexDataSourceProvider(defaultDataSourceCreator, dynamicDataSourceProperties, master.getUrl(),
				master.getUsername(), master.getPassword());
	}*/

	/**
	 * 动态数据源创建类
	 * 
	 * @param dataSourceCreator
	 * @return
	 */
/*	@Bean
	public DefaultDataSourceCreator defaultDataSourceCreator(DruidDataSourceCreator dataSourceCreator) {
		DefaultDataSourceCreator defaultDataSourceCreator = new DefaultDataSourceCreator();
		defaultDataSourceCreator.setCreators(Arrays.asList(dataSourceCreator));
		return defaultDataSourceCreator;
	}*/

	/**
	 * 神策埋点接口
	 * @return
	 * @throws IOException
	 * @throws InvalidArgumentException 
	 */
	@Bean(destroyMethod = "shutdown")
	@ConditionalOnProperty(value = "blade.analytics.enabled", havingValue = "true")
	public ISensorsAnalytics initSensorsAnalytics() throws Exception {
//		String project = "bi", projectType = "bi";
//		String urlArgs = String.format("project=%s&ptype=%s", project, projectType);
//		String serviceUrl = analyticsProperties.getServiceUrl();
//		serviceUrl += (serviceUrl.contains("?") ? "&" : "?") + urlArgs;
		final FastBatchConsumer fastBatchConsumer = new FastBatchConsumer(analyticsProperties.getServiceUrl(), false, 
				10, //bulkSize - 触发 flush 操作阈值，当内存缓存队列达到该值时，将缓存中的数据批量上报，默认 50
				6000,  //maxCacheSize - 内存缓存队列保存记录最大条数，默认 6000，上限 10000，下限 1000
				1,  //flushSec - 消费者线程定时执行频率，默认 1S
				3,  //timeoutSec - 网络请求超时时间，默认 3S
				failedData -> { //回调函数返回失败数据
					log.warn("sensor:track:failed|" + JSON.toJSONString(failedData));
				}
		);
		// 构建 sa 实例对象
		final SensorsAnalytics sa = new SensorsAnalytics(fastBatchConsumer);
		log.info("初始化神策埋点完成......");
		//设置公共属性,以后上传的每一个事件都附带该属性
//        SuperPropertiesRecord superProperties = SuperPropertiesRecord.builder()
//                .addProperty("$title", "Windows")
//                .build();
//        sa.registerSuperProperties(superProperties);
		return sa;
	}
}
