/**
 *
 */
package org.springblade.cube.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.enums.FunctionAttrEnum;
import org.springblade.cube.enums.FunctionTypeEnum;
import org.springblade.cube.model.CubeDatasouceFunction;
import org.springblade.cube.service.CubeDatasouceFunctionService;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * @author wookvn
 */
@RestController
@RequestMapping("/cube/function")
@Api(value = "数据源函数接口", tags = "数据源函数相关接口")
public class CubeDatasouceFunctionController extends BladeController {

    @Autowired
    private CubeDatasouceFunctionService cubeDatasouceFunctionService;

    /**
     * @param dsType
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "函数列表")
    public R<Page<CubeDatasouceFunction>> list(@RequestParam DsTypeEnum dsType, FunctionAttrEnum funcAttr, String name,
                                               @RequestParam(defaultValue = "1") int page,
                                               @RequestParam(defaultValue = "10") int size) {
        QueryWrapper<CubeDatasouceFunction> queryWrapper = Wrappers.<CubeDatasouceFunction>query();
//		queryWrapper.in("ds_type", dsType.getValue(), -1); //数据源类型【-1 代表通用】
        queryWrapper.lambda()
                .in(CubeDatasouceFunction::getDsType, Arrays.asList(dsType, DsTypeEnum.common))
                .like(StrUtil.isNotBlank(name), CubeDatasouceFunction::getName, name)
                .eq(funcAttr != null, CubeDatasouceFunction::getFuncAttr, funcAttr)
                .eq(CubeDatasouceFunction::getTenantId, getUser().getTenantId())
//				.and(t -> t.eq(CubeDatasouceFunction::getFuncType, FunctionTypeEnum.system).or().eq(CubeDatasouceFunction::getTenantId,getUser().getTenantId()))
                .orderByDesc(CubeDatasouceFunction::getSortNum);
        Page<CubeDatasouceFunction> datasouceFunctionPage = cubeDatasouceFunctionService.page(new Page<>(page, size), queryWrapper);
        return R.data(datasouceFunctionPage);
    }

    /**
     * @return
     */
    @PostMapping("/submit")
    @ApiOperation(value = "新增修改函数")
    public R<String> doSubmit(@RequestBody CubeDatasouceFunction datasourceFunction) {
        //判重
        List<CubeDatasouceFunction> list = cubeDatasouceFunctionService.lambdaQuery()
                .eq(CubeDatasouceFunction::getDsType, datasourceFunction.getDsType())
                .ne(datasourceFunction.getFuncId() != null, CubeDatasouceFunction::getFuncId, datasourceFunction.getFuncId())
                .eq(CubeDatasouceFunction::getName, datasourceFunction.getName()).list();
        if (CollectionUtil.isNotEmpty(list)) {
            return R.fail("函数名称重复");
        }
        List<CubeDatasouceFunction> list1 = cubeDatasouceFunctionService.lambdaQuery()
                .eq(CubeDatasouceFunction::getDsType, datasourceFunction.getDsType())
                .ne(datasourceFunction.getFuncId() != null, CubeDatasouceFunction::getFuncId, datasourceFunction.getFuncId())
                .eq(CubeDatasouceFunction::getMethod, datasourceFunction.getMethod()).list();
        if (CollectionUtil.isNotEmpty(list1)) {
            return R.fail("函数方法重复");
        }
        datasourceFunction.setCreateTime(new Date());
        boolean ret = cubeDatasouceFunctionService.saveOrUpdate(datasourceFunction);
        //刷新缓存
        if(ret) {
            cubeDatasouceFunctionService.refreshCache(AuthUtil.getTenantId(), datasourceFunction.getDsType());
        }
        return R.status(ret);
    }

    /**
     * @return
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除函数")
    public R<String> doDelete(@ApiParam(value = "主键", required = true) @RequestParam String ids) {
        List<Long> funcIdList = Func.toLongList(ids);
        List<CubeDatasouceFunction> cubeDatasouceFunctions = cubeDatasouceFunctionService.listByIds(funcIdList);
        Set<DsTypeEnum> dsTypeEnums = new HashSet<>();
        for(CubeDatasouceFunction dsFunction : cubeDatasouceFunctions) {
            dsTypeEnums.add(dsFunction.getDsType());
        }
        boolean ret = cubeDatasouceFunctionService.removeByIds(funcIdList);
        if(ret){
            dsTypeEnums.forEach(dsType -> cubeDatasouceFunctionService.refreshCache(AuthUtil.getTenantId(), dsType));
        }
        return R.status(ret);
    }

    /**
     * @return
     */
    @GetMapping("/funcAttrList")
    @ApiOperation(value = "函数属性列表")
    public R<List<Kv>> funcAttrList() {
        List<Kv> dataList = new ArrayList<>();
        for (FunctionAttrEnum funcAttr : FunctionAttrEnum.values()) {
            dataList.add(Kv.create().set("value", funcAttr.getValue()).set("name", funcAttr.name()).set("message", funcAttr.getMessage()));
        }
        return R.data(dataList);
    }
}
