package org.springblade.cube.controller;

import java.time.LocalDateTime;
import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.cube.model.CubeGlobalParam;
import org.springblade.cube.model.CubeGlobalParamType;
import org.springblade.cube.service.CubeGlobalParamService;
import org.springblade.cube.service.CubeGlobalParamTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 可视化分类表 控制器
 *
 * @author BladeX
 * @since 2021-03-05
 */
@RestController
@RequestMapping("cube/globalParamType")
@Api(value = "数据集全局参数分类接口", tags = "数据集全局参数分类接口")
public class CubeGlobalParamTypeController extends BladeController {

    @Autowired
    private CubeGlobalParamTypeService cubeGlobalParamTypeService;

    @Autowired
    private CubeGlobalParamService cubeGlobalParamService;

    /**
     * 列表数据
     *
     * @param cubeGlobalParamType
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询", httpMethod = "GET")
    public R<IPage<CubeGlobalParamType>> pageList(CubeGlobalParamType cubeGlobalParamType, Query query) {
        QueryWrapper<CubeGlobalParamType> queryWrapper = Condition.getQueryWrapper(cubeGlobalParamType);
        return R.data(cubeGlobalParamTypeService.page(Condition.getPage(query), queryWrapper.lambda().orderByDesc(CubeGlobalParamType::getSortNum)));
    }

    /**
     * 修改数据接口
     *
     * @param cubeGlobalParamType
     * @return
     */
    @PostMapping("/submit")
    @ApiOperation(value = "增加或修改", httpMethod = "POST")
    public R<String> doUpdate(@Validated @RequestBody CubeGlobalParamType cubeGlobalParamType) {
        //判重
        List<CubeGlobalParamType> list = cubeGlobalParamTypeService.lambdaQuery()
                .eq(CubeGlobalParamType::getTypeName, cubeGlobalParamType.getTypeName())
                .eq(CubeGlobalParamType::getParentId,cubeGlobalParamType.getParentId())
                .ne(cubeGlobalParamType.getId() != null, CubeGlobalParamType::getId, cubeGlobalParamType.getId()).list();
        if (CollectionUtil.isNotEmpty(list)) {
            return R.fail("名字重复");
        }
        if (cubeGlobalParamType.getId() == null) {
            cubeGlobalParamType.setCreateTime(LocalDateTime.now());
            cubeGlobalParamType.setCreateUserId(getUser().getUserId());
        }
        return R.status(cubeGlobalParamTypeService.saveOrUpdate(cubeGlobalParamType));
    }

    /**
     * 删除数据接口
     *
     * @param id
     * @return
     */
    @PostMapping("/delete")
    @ApiOperation(value = "根据ID删除", httpMethod = "POST")
    public R<String> doDelete(@ApiParam(value = "主键", required = true) @RequestParam Long id) {
        long count = cubeGlobalParamTypeService.count(Wrappers.<CubeGlobalParamType>lambdaQuery().eq(CubeGlobalParamType::getParentId, id));
        if (count > 0) {
            return R.fail("该分类下有子分类，不能删除。");
        }
        long visualCount = cubeGlobalParamService.count(Wrappers.<CubeGlobalParam>lambdaQuery().eq(CubeGlobalParam::getTypeId, id));
        if (visualCount > 0) {
            return R.fail("该分类下有数据，不能删除。");
        }
        return R.status(cubeGlobalParamTypeService.removeById(id));
    }

}
