/**
 * 
 */
package org.springblade.cube.service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.constant.CacheConstants;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.cube.enums.FunctionTypeEnum;
import org.springblade.cube.mapper.CubeDatasouceFunctionMapper;
import org.springblade.cube.model.CubeDatasouceFunction;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.util.List;

/**
 * @author wookvn
 *
 */
@Service
public class CubeDatasouceFunctionService extends ServiceImpl<CubeDatasouceFunctionMapper, CubeDatasouceFunction> {

    @Autowired
    private BladeRedis bladeRedis;

    /**
     * <p>查询自定义的函数</p>
     * @author wookvn
     * @datetime 2024-11-06 10:47:45
     * @return java.util.List<org.springblade.cube.model.CubeDatasouceFunction>
     **/
    public List<CubeDatasouceFunction> listCustomFunction(String tenantId, DsTypeEnum dsType) {
        String key = CacheConstants.DATASOURCE_FUNCTION_CACHE_KEY_PREFIX + tenantId + ":" + dsType;
        return bladeRedis.get(key,() -> super.list(Wrappers.<CubeDatasouceFunction>lambdaQuery().eq(CubeDatasouceFunction::getDsType, dsType)
                .eq(CubeDatasouceFunction::getFuncType, FunctionTypeEnum.custom).eq(CubeDatasouceFunction::getTenantId, tenantId)));
    }

    /**
     * <p>刷新缓存</p>
     * @author wookvn
     * @datetime 2024-11-06 10:52:20
     * @param tenantId: 租户ID
     * @param dsType: 数据源类型
     **/
    public void refreshCache(String tenantId, DsTypeEnum dsType) {
        String key = CacheConstants.DATASOURCE_FUNCTION_CACHE_KEY_PREFIX + tenantId + ":" + dsType;
        bladeRedis.del(key);
    }
}
