/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.validation.Valid;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.io.FilenameUtils;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Base62Util;
import org.springblade.core.tool.utils.Func;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.oss.bean.OssResult;
import org.springblade.oss.handler.OssHandler;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.entity.ReportDatasourcePermission;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.report.service.normal.ReportDatasourcePermissionService;
import org.springblade.report.vo.PublicDsParams;
import org.springblade.system.client.DataPermClient;
import org.springblade.system.client.IamClient;
import org.springblade.system.client.dto.IamGetUserByIdsParams;
import org.springblade.system.client.model.UserModel;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

/**
 * 报表数据源 控制器
 *
 * @since 2021-03-22
 */
@Slf4j
@RestController
@RequestMapping("report/reportdatasource")
@Api(value = "报表数据源", tags = "报表数据源 DataSourceControler")
public class ReportDatasourceController extends BladeController {

	@Autowired
	private IReportDatasourceService reportDatasourceService;

	@Autowired
	private DynamicDataSourceSupport dynamicDataSourceSupport;

	@Autowired
	private ReportDatasourcePermissionService reportDatasourcePermissionService;

	@Autowired
	private DataPermClient dataPermClient;

	@Autowired
	private OssHandler ossHandler;

	@Autowired
	private DialectContext dialectContext;

	@Autowired
	private IamClient iamClient;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情 detail", notes = "传入reportDatasource")
	public R<ReportDatasource> detail(ReportDatasource reportDatasource) {
		ReportDatasource detail = reportDatasourceService.getOne(Condition.getQueryWrapper(reportDatasource));
		detail.setDsPassword(null);
		return R.data(detail);
	}

	/**
	 * 分页 报表数据源
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表 list", notes = "传入reportDatasource")
	@Deprecated
	public R<List<ReportDatasource>> list(ReportDatasource reportDatasource) {
		Long userId = getUser().getUserId();
		LambdaQueryWrapper<ReportDatasource> queryWrapper = Condition.getQueryWrapper(reportDatasource).lambda().orderByDesc(ReportDatasource::getCreateTime);
		queryWrapper.eq(ReportDatasource::getWhetherPublic,0);
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
		if(checkSuperPermRet == null || !checkSuperPermRet) {
			queryWrapper.and(t ->{
				t.eq(ReportDatasource::getCreateUserId, userId)
				.or().inSql(ReportDatasource::getId, "SELECT a.datasource_id FROM report_datasource_permission a INNER JOIN data_permission_user b ON a.data_perm_id=b.data_perm_id WHERE b.user_id=" + userId);
			});
		}
		queryWrapper.eq(ReportDatasource::getIsEdit,1);
		List<ReportDatasource> dataList = reportDatasourceService.list(queryWrapper);
		if(CollectionUtil.isNotEmpty(dataList)) {
			dataList.forEach(item -> item.setDsPassword(null));
		}
		return R.data(dataList);
	}

	/**
	 * 自定义分页 报表数据源
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页 page", notes = "传入reportDatasource")
	public R<IPage<ReportDatasource>> page(ReportDatasource reportDatasource, Query query) {
		Long userId = getUser().getUserId();
		LambdaQueryWrapper<ReportDatasource> queryWrapper = Condition.getQueryWrapper(reportDatasource).lambda().orderByDesc(ReportDatasource::getCreateTime);
		queryWrapper.eq(ReportDatasource::getWhetherPublic,0);
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
		if(checkSuperPermRet == null || !checkSuperPermRet) {
			queryWrapper.and(t ->{
				t.eq(ReportDatasource::getCreateUserId, userId)
				.or().inSql(ReportDatasource::getId, "SELECT a.datasource_id FROM report_datasource_permission a INNER JOIN data_permission_user b ON a.data_perm_id=b.data_perm_id WHERE b.user_id=" + userId);
			});
		}
		IPage<ReportDatasource> page = reportDatasourceService.page(Condition.getPage(query), queryWrapper);
		if(page != null && CollectionUtil.isNotEmpty(page.getRecords())) {
			List<Long> userIds = page.getRecords().stream().map(ReportDatasource::getResponsiblePersonId).collect(Collectors.toList());
			List<UserModel> userModelList = iamClient.getByIds(new IamGetUserByIdsParams(userIds, AuthUtil.getInstId(), 0));
			Map<Long, String> nickNameMap = new HashMap<>();
			if (org.springblade.core.tool.utils.CollectionUtil.isNotEmpty(userModelList)) {
				nickNameMap = userModelList.stream().collect(Collectors.toMap(UserModel::getId, UserModel::getNickName));
			}
            for (ReportDatasource record : page.getRecords()) {
                record.setDsPassword(null);
                String nickName = nickNameMap.get(record.getResponsiblePersonId());
                if (StrUtil.isNotBlank(nickName)){
                    record.setResponsiblePersonName(nickName);
                }
            }
		}
		return R.data(page);
	}

	/**
	 * 新增 报表数据源
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增 save", notes = "传入reportDatasource")
	public R<?> save(@Valid @RequestBody ReportDatasource dto) {
		//判断数据源名称是否重复
		long count = reportDatasourceService.count(Wrappers.<ReportDatasource>lambdaQuery().eq(ReportDatasource::getDsName, dto.getDsName()));
		if(count > 0) {
			return R.fail("数据源名称重复");
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(dto.getDsType());
		dto.setDriverClassName(dsTypeEnum.getDriverClassName());
		dto.setUrl(dsTypeEnum.getDriverUrlFunction().apply(dto));
		dto.setCreateUserId(getUser().getUserId());
		dto.setDbSchema(StrUtil.isBlank(dto.getDbSchema()) ? dialectContext.getDialect(dsTypeEnum).dbSchema(dto) : dto.getDbSchema());
		//设置默认poolName  UUID.randomUUID().toString().replaceAll("-", "").substring(0, 10)
		String poolName = Base62Util.encode(IdUtil.getSnowflakeNextId());
		dto.setPoolName(poolName);
		//检测数据源是否有效
		String errMsg = DynamicDataSourceSupport.testDs(dto);
		if(errMsg != null) {
			return R.fail("数据源连接失败，请检查参数");
		}
		return R.status(reportDatasourceService.save(dto));
	}

	/**
	 * 修改 报表数据源
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改 update", notes = "传入reportDatasource")
	public R<?> update(@Valid @RequestBody ReportDatasource dto) {
		if(dto.getId() == null || dto.getId() <= 0) {
			return R.fail("数据源ID不能为空");
		}
		//判断数据源名称是否重复
		long count = reportDatasourceService.count(Wrappers.<ReportDatasource>lambdaQuery()
				.eq(ReportDatasource::getDsName, dto.getDsName()).ne(ReportDatasource::getId, dto.getId()));
		if(count > 0) {
			return R.fail("连接名重复");
		}
		ReportDatasource dbReportDs = reportDatasourceService.getById(dto.getId());
		if(dbReportDs == null) {
			return R.fail("数据源不存在");
		}
		if(StrUtil.isBlank(dto.getDsPassword())) {
			dto.setDsPassword(dbReportDs.getDsPassword());
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(dto.getDsType());
		dto.setDriverClassName(dsTypeEnum.getDriverClassName());
		dto.setUrl(dsTypeEnum.getDriverUrlFunction().apply(dto));
		dto.setDbSchema(StrUtil.isBlank(dto.getDbSchema()) ? dialectContext.getDialect(dsTypeEnum).dbSchema(dto) : dto.getDbSchema());
		try {
			//先删除，再添加
			dto.setPoolName(Base62Util.encode(IdUtil.getSnowflakeNextId()));
			dto.setDsPassword(StrUtil.isBlank(dto.getDsPassword()) ? null : dbReportDs.getDsPassword());
			//检测数据源是否有效
			String errMsg = DynamicDataSourceSupport.testDs(dto);
			if(errMsg != null) {
				return R.fail("数据源连接失败，请检查参数");
			}
			//刷新缓存
			reportDatasourceService.refreshCache(dbReportDs.getId(), dbReportDs.getPoolName());
			return R.status(reportDatasourceService.updateById(dto));
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return R.fail("修改数据源失败");
		}
	}

	/**
	 * 删除 报表数据源
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "删除 remove", notes = "传入ids")
	public R<?> remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		List<Long> idList = Func.toLongList(ids);
		List<ReportDatasource> reportDsList = reportDatasourceService.listByIds(idList);
		for(ReportDatasource dto : reportDsList) {  //先移除数据源，再删除报表
			try {
				//刷新缓存
				reportDatasourceService.refreshCache(dto.getId(), dto.getPoolName());
			} catch (Exception e) {
				log.error(e.getMessage(), e);
			}
		}
		List<Long> dsIds = Func.toLongList(ids);
		reportDatasourceService.removeByIds(dsIds);
		reportDatasourcePermissionService.remove(Wrappers.<ReportDatasourcePermission>lambdaQuery().in(ReportDatasourcePermission::getDatasourceId, dsIds));
		return R.status(true);
	}

	/**
	 * 测试数据源
	 * @param dto
	 * @return
	 */
	@PostMapping("/testDs")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "测试数据源 testDs", notes = "传入数据源信息")
	public R<?> testDs(@Validated @RequestBody ReportDatasource dto) {
		//当新增时，密码不能为空
		if(dto.getId() == null && StrUtil.isBlank(dto.getDsPassword())) {
			return R.fail("数据源连接密码不能为空");
		}
		//当更新时，密码为空，需要从数据库查询密码
		if(dto.getId() != null && StrUtil.isBlank(dto.getDsPassword())) {
			ReportDatasource dbReportDs = reportDatasourceService.getById(dto.getId());
			if(dbReportDs != null) {
				dto.setDsPassword(dbReportDs.getDsPassword());
			}
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(dto.getDsType());
		dto.setUrl(dsTypeEnum.getDriverUrlFunction().apply(dto));
		String errMsg = DynamicDataSourceSupport.testDs(dto);
		if(errMsg == null) {
			return R.success("数据源连接成功");
		}else {
			R<?> ret = R.fail("数据源连接失败，请检查参数");
			ret.setExt(errMsg);
			return ret;
		}
	}

	@PostMapping("/uploadJar")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "上传JDBC驱动jar", notes = "返回jar包的路径")
	public R<String> uploadJar(MultipartFile file) throws Exception {
		String extension = file != null ? FilenameUtils.getExtension(file.getOriginalFilename()) : "";
		if(!"jar".equalsIgnoreCase(extension)) {
			return R.fail("jar包格式不正确");
		}
		String tempFileName = UUID.randomUUID().toString() + "." + extension;
		OssResult ossResult = ossHandler.uploadByBytes("jdbcdriver/" + tempFileName, file.getBytes());
		if(!ossResult.isOk()) {
			log.error("上传jar包失败，失败原因：" + ossResult.getMessage());
			return R.fail("上传jar包失败");
		}
//		File jarFile = new File(uploadJdbcDriverJarPath + tempFileName);
//		File jarFileFolder = jarFile.getParentFile();
//		if(!jarFileFolder.exists()) {
//			jarFileFolder.mkdirs();
//		}
//		FileUtils.copyInputStreamToFile(file.getInputStream(), jarFile);
		return R.data(ossResult.getEndpoint() + ossResult.getFileUri());
	}

	@PostMapping("/refreshCache")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "刷新数据库表的缓存[支持批量刷新]")
	public R<String> refreshCache(@RequestBody List<Long> dsIdList) {
		dialectContext.refreshDsCache(dsIdList);
		return R.success("缓存刷新完成");
	}

	@GetMapping("/publicDs")
	@ApiOperation(value = "公共数据源")
	public R<Page<ReportDatasource>> publicDs(PublicDsParams params){
		return R.data(reportDatasourceService.publicDs(params));
	}

}
