/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.mapper;

import java.util.List;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.report.entity.ReportDatasource;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.springblade.report.vo.PublicDsParams;

/**
 * 报表数据源 Mapper 接口
 *
 * @author BladeX
 * @since 2021-03-22
 */
@DS("master")
public interface ReportDatasourceMapper extends BaseMapper<ReportDatasource> {

    @InterceptorIgnore(tenantLine = "true")
    @Select("select * from report_datasource where tenant_id = #{tenantId}")
    List<ReportDatasource> getList(String tenantId);

    @InterceptorIgnore(tenantLine = "true")
    @Select("select * from report_datasource where tenant_id = #{tenantId} and id = #{datasourceId}")
    ReportDatasource getByIdAndTenantId(Long datasourceId, String tenantId);
    
    @InterceptorIgnore(tenantLine = "true")
    @Select("select * from report_datasource where pool_name = #{value}")
    ReportDatasource selectByPoolName(String poolName);

    @InterceptorIgnore(tenantLine = "true")
    @Select({
            "<script>",
            "SELECT * ",
            "FROM report_datasource ",
            "WHERE whether_public = 1 ",
            "<if test='params.dsName != null and params.dsName != \"\"'>",
            "  AND ds_name LIKE CONCAT('%', #{params.dsName}, '%') ",
            "</if>",
            "<if test='params.dsType != null'>",
            "  AND ds_type = #{params.dsType} ",
            "</if>",
            "ORDER BY create_time DESC",
            "</script>"
    })
    Page<ReportDatasource> publicDs(Page<ReportDatasource> page, @Param("params") PublicDsParams params);

    @InterceptorIgnore(tenantLine = "true")
    @Select("SELECT * " +
            "FROM report_datasource " +
            "WHERE whether_public = 1 ")
    List<ReportDatasource> allPublicDs();

}
