/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.redis.cache.CacheKey;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.mapper.ReportDatasourceMapper;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.constant.CacheConstants;
import org.springblade.report.vo.PublicDsParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.time.Duration;
import java.util.List;

/**
 * 报表数据源 服务实现类
 *
 * @author BladeX
 * @since 2021-03-22
 */
@Service
public class ReportDatasourceServiceImpl extends ServiceImpl<ReportDatasourceMapper, ReportDatasource> implements IReportDatasourceService {

    @Autowired
    private BladeRedis bladeRedis;

    @Override
    public List<ReportDatasource> getByTenantId(String tenantId) {
        List<ReportDatasource> list = baseMapper.getList(tenantId);
        if (CollectionUtil.isNotEmpty(list)) {
            list.forEach(reportDatasource -> reportDatasource.setDsPassword(null));
        }
        return list;
    }

    @Override
    public ReportDatasource getByIdAndTenantId(Long datasourceId, String tenantId) {
        return baseMapper.getByIdAndTenantId(datasourceId,tenantId);
    }

    @Override
    public ReportDatasource getById(Serializable id) {
        String cacheKey = CacheConstants.DS_ID_CAHCE_KEY_PREFIX + id;
        return bladeRedis.get(new CacheKey(cacheKey, Duration.ofSeconds(CacheConstants.DS_ID_CAHCE_TIMEOUT_SEC)), () -> baseMapper.selectById(id));
    }

    @Override
	public ReportDatasource getCacheByPoolName(String poolName) {
        String cacheKey = CacheConstants.DS_POOL_NAME_CAHCE_KEY_PREFIX + poolName;
        return bladeRedis.get(new CacheKey(cacheKey, Duration.ofSeconds(CacheConstants.DS_POOL_NAME_CAHCE_TIMEOUT_SEC)), () -> baseMapper.selectByPoolName(poolName));
	}

    @Override
    public void refreshCache(Long dsId, String poolName) {
        String idCacheKey = CacheConstants.DS_ID_CAHCE_KEY_PREFIX + dsId,
                poolNameCacheKey = CacheConstants.DS_POOL_NAME_CAHCE_KEY_PREFIX + poolName;
        bladeRedis.del(idCacheKey, poolNameCacheKey);
    }

    @Override
    public Page<ReportDatasource> publicDs(PublicDsParams publicDsParams) {
        return this.baseMapper.publicDs(new Page<>(publicDsParams.getCurrent(), publicDsParams.getSize()),publicDsParams);
    }


}
