package org.springblade.visual.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson2.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.cube.dto.SelectParams;
import org.springblade.visual.dto.VisualLoginFreeLinkPermissionDoo;
import org.springblade.visual.dto.VisualLoginLinkPermissionDto;
import org.springblade.visual.entity.VisualLoginFreeLinkPermission;
import org.springblade.visual.service.VisualLoginFreeLinkPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author: hepingguo
 * @Description:
 * @Date: 2023/3/20 10:41
 **/
@RestController
@RequestMapping("visual/longinfreelink/permission")
@Api(value = "免登陆链接接口", tags = "免登陆链接权限组接口")
public class VisualLoginFreeLinkPermissionController extends BladeController {

    @Autowired
    private VisualLoginFreeLinkPermissionService visualLoginFreeLinkPermissionService;

    @GetMapping("/login/free/links")
    @ApiOperation(value = "根据权限组ID查询免登录链接ID列表")
    public R<List<Long>> visualIdList(@RequestParam Long dataPermId){
        return R.data(visualLoginFreeLinkPermissionService.listLoginFreeLinkId(dataPermId));
    }

    @PostMapping("/select/{dataPermId}")
    @ApiOperation(value = "已选择的id")
    public R<List<Long>> selectCubeIdList(@RequestBody List<Long> ids, @PathVariable Long dataPermId) {
        List<VisualLoginFreeLinkPermission> list = visualLoginFreeLinkPermissionService.lambdaQuery().eq(VisualLoginFreeLinkPermission::getDataPermId, dataPermId).in(VisualLoginFreeLinkPermission::getLinkId, ids).list();
        return R.data(CollectionUtil.isNotEmpty(list) ? list.stream().map(VisualLoginFreeLinkPermission::getLinkId).collect(Collectors.toList()) : new ArrayList<>());
    }

    /**
     * 权限组管理-免登录链接设置的入口
     * @param linkPermissionDoo
     * @return
     */
    @PostMapping("/add/perm")
    @ApiOperation(value = "添加或修该权限组与免登录链接的关系", notes = "权限组管理-免登录链接设置的入口")
    public R<String> doSubmit(@RequestBody VisualLoginFreeLinkPermissionDoo linkPermissionDoo) {
        return R.status(visualLoginFreeLinkPermissionService.doSaveOrUpdate(linkPermissionDoo, getUser()));
    }

    @ApiOperation("免登录链接选中")
    @PostMapping("/selectSubmit")
    public R<Boolean> selectSubmit(@RequestBody SelectParams params) {
        return R.status(visualLoginFreeLinkPermissionService.selectSubmit(params));
    }

    @PostMapping("/checkPermission/{linkId}/{userId}")
    @ApiOperation(value = "判断用户是否有免登录链接权限")
    public R<Boolean> checkPermission(@PathVariable Integer linkId, @PathVariable Long userId){
        BladeUser bladeUser = new BladeUser();
        bladeUser.setUserId(userId);
        return R.data(visualLoginFreeLinkPermissionService.checkVisualDataForUser(linkId, 0L, bladeUser));
    }

    @GetMapping("/dataPermList/{linkId}")
    @ApiOperation(value = "根据免登录链接ID查询权限组列表")
    public R<List<JSONObject>> dataPermList(@PathVariable Long linkId){
        return R.data(visualLoginFreeLinkPermissionService.listDataPerm(linkId));
    }




    /**
     * 免登录链接列表权限组管理入口
     * @param linkPermissionDto
     * @return
     */
    @PostMapping("/perform")
    @ApiOperation(value = "批量添加或修改权限组与免登录链接关系", notes = "免登录链接列表-权限组管理入口【超级管理员才能操作】")
    public R<String> doPerform(@RequestBody VisualLoginLinkPermissionDto linkPermissionDto) {
        if(!AuthUtil.isAdmin()) {
            return R.fail("只能超级管理员操作");
        }
        return R.status(visualLoginFreeLinkPermissionService.doPerform(linkPermissionDto, getUser()));
    }
}
