package org.springblade.visual.controller;

import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.constant.CacheConstants;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.redis.cache.CacheKey;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.visual.dto.*;
import org.springblade.visual.service.ComponentStatisticsService;
import org.springblade.visual.service.VisualTemplateStatisticsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.time.Duration;
import java.util.List;

@Api(tags = "大屏模板统计接口")
@RestController
@RequestMapping("visual/template/statistics")
public class VisualTemplateStatisticsController {

    @Autowired
    private VisualTemplateStatisticsService visualTemplateStatisticsService;

    @Autowired
    private BladeRedis bladeRedis;

    /**
     * 折线图
     */
    @GetMapping("/templateLineChart")
    @ApiOperation("折线图")
    public R lineChart(VisualTemplateStatisticsParams params) {
        String parmJson = JSONUtil.toJsonStr(params);
        String cacheKey = CacheConstants.VISUAL_TEMPLATE_STAT_LINECHART_CACHE_KEY_PREFIX + DigestUtil.md5Hex(parmJson + AuthUtil.getTenantId());
        return R.data(bladeRedis.get(new CacheKey(cacheKey, Duration.ofSeconds(CacheConstants.VISUAL_TEMPLATE_STAT_CACHE_TIMEOUT_SEC)), () -> visualTemplateStatisticsService.lineChart(params)));
    }

    /**
     * 列表
     */
    @GetMapping("/templateList")
    @ApiOperation("列表")
    public R templateList(VisualTemplateStatisticsParams params) {
        return R.data(visualTemplateStatisticsService.templateList(params));
    }

    @GetMapping("/templateListDownload")
    @ApiOperation("大屏模板使用统计导出")
    public void templateListDownload(VisualTemplateStatisticsParams params, HttpServletResponse response){
        visualTemplateStatisticsService.templateListDownload(params, response);
    }

    /**
     * 模板top
     */
    @GetMapping("/templateTop")
    @ApiOperation("模板top")
    public R componentTop(
            @ApiParam("top多少")
            @RequestParam(defaultValue = "10") Integer top) {
        String cacheKey = CacheConstants.VISUAL_TEMPLATE_STAT_TOPN_CACHE_KEY_PREFIX + top + ":" + AuthUtil.getTenantId();
        return R.data(bladeRedis.get(new CacheKey(cacheKey, Duration.ofSeconds(CacheConstants.VISUAL_TEMPLATE_STAT_CACHE_TIMEOUT_SEC)), () -> visualTemplateStatisticsService.templateTop(top)));
    }
}
