/**
 * 
 */
package org.springblade.visual.entity;

import java.io.Serializable;
import java.util.Date;

import com.baomidou.mybatisplus.annotation.TableField;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springblade.visual.enums.ComponentUseTypeEnum;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.NullSerializer;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * @author wookvn
 *
 */
@Data
@TableName("visual_component")
@ApiModel(value = "VisualComponent对象", description = "模板组件表")
public class VisualComponent extends TenantEntity implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@TableId
	private Long id;
	
	@ApiModelProperty("图表类型标志") //比如：new-bar,new-table
	private String name;
	
	@ApiModelProperty("图层名称") //前端使用
	private String title;

	@ApiModelProperty("可视化主键ID")
	private Long visualId;

	@TableField(exist = false)
	@ApiModelProperty("可视化大屏名字")
	private String visualName;
	
	@ApiModelProperty("数据源ID")
	private Long datasourceId;
	
	@ApiModelProperty("数据集ID")
	private Long cubeDataId;
	
	@ApiModelProperty("查询条件JSON")
	private String filterJson;
	
	@ApiModelProperty("查询条件关系JSON")
	private String filterRelationJson;
	
	@ApiModelProperty("字段JSON")
	private String columnJson;
	
	@ApiModelProperty("排序JSON")
	private String sortJson;
	
	@ApiModelProperty("额外JSON")
	private String extJson;
	
	@JsonIgnore
	@ApiModelProperty(hidden = true)
	private String querySql;
	
	@ApiModelProperty("使用方式(0-数据集/1-API)")
	@JsonSerialize(nullsUsing = NullSerializer.class)
	private ComponentUseTypeEnum useType;
	
	@ApiModelProperty("显示条数")
	private Integer limitVal;
	
	@ApiModelProperty("创建时间")
	private Date createTime;
	
	@ApiModelProperty("创建人")
	private String createBy;
	
	@ApiModelProperty("创建人ID")
	@JsonSerialize(using = ToStringSerializer.class)
	private Long createUserId;
	
	@ApiModelProperty(value = "组件内容")
	private String content;
	
	@ApiModelProperty(value = "组件类型")
	@JsonSerialize(nullsUsing = NullSerializer.class)
	private ComponentTypeEnum compType;
	@ApiModelProperty(value = "组件来源 0是系统组件 1是超管自定义组件 2用户自定义组件 3是组件模板-公共 4是组件模板-私有")
	private Integer componentSource;

	@ApiModelProperty(value = "组件来源id")
	private Long componentSourceId;
	
	@JsonIgnore
	@ApiModelProperty(hidden = true)
	private String columnIdList;

	@ApiModelProperty("是否为移动大屏组件")
	private Integer isMobile;

	@ApiModelProperty("是否静态")
	private Integer whetherStatic;
	@ApiModelProperty("静态数据地址")
	private String staticDataPath;

	@TableField(exist = false)
	@ApiModelProperty("租户名字")
	private String tenantName;

	@TableField(exist = false)
	@ApiModelProperty("组件模板创建人")
	private String componentTemplateCreateUser;

	@TableField(exist = false)
	@ApiModelProperty("组件模板分类")
	private String componentTemplateCategory;
	
//	@ApiModelProperty("API接口Key")
//	private String apiKey;
	
}
