package org.springblade.visual.mapper;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.microapp.dto.MicroAppViewCountDto;
import org.springblade.visual.dto.MobileStatisticsReturnVO;
import org.springblade.visual.dto.VisualViewCountDto;

import java.util.List;

@DS("clickhouse")
@InterceptorIgnore(tenantLine = "true")
@Mapper
public interface CkMapper {

    /**
     * 通过id查找大屏或者报表访问量
     */
    @Select({
            "<script>",
            "SELECT attr8 as visualId, COUNT() as viewCount",
            "FROM bi_event",
            "WHERE event = 'visualPreview'",
            "AND attr1 = #{type}",
            "<if test='ids != null and ids.size() > 0'>",
            "AND attr8 IN ",
            "<foreach item='id' index='index' collection='ids' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>",
            "</if>",
            "GROUP BY attr8",
            "</script>"
    })
    List<VisualViewCountDto> getVisualViewCount(List<Long> ids, String type);



    /**
     * 通过id查找应用访问量
     */
    @Select({
            "<script>",
            "SELECT attr2 as microAppId, COUNT() as viewCount",
            "FROM bi_event",
            "WHERE event = 'appPreview'",
            "<if test='ids != null and ids.size() > 0'>",
            "AND attr2 IN ",
            "<foreach item='id' index='index' collection='ids' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>",
            "</if>",
            "GROUP BY attr2",
            "</script>"
    })
    List<MicroAppViewCountDto> getMicroAppViewCount(List<Long> ids);


    /**
     * 大屏排序 限制个数
     */
    @Select({
            "<script>",
            "SELECT attr8 AS visualId,",
            "       COUNT(*) AS visitCount,",
            "       attr2 AS visualName",
            "FROM bi_event",
            "WHERE event = 'visualPreview'",
            "<if test='types != null and types.size() > 0'>",
            "  AND attr1 IN ",
            "  <foreach item='type' index='index' collection='types' open='(' separator=',' close=')'>",
            "    #{type}",
            "  </foreach>",
            "</if>",
            "<if test='ids != null and ids.size() > 0'>",
            "  AND attr8 IN ",
            "  <foreach item='id' index='index' collection='ids' open='(' separator=',' close=')'>",
            "    #{id}",
            "  </foreach>",
            "</if>",
            "GROUP BY attr8, attr2",
            "ORDER BY visitCount DESC",
            "<if test='limit != null'>", // 添加对 LIMIT 的判断
            "  LIMIT #{limit}",
            "</if>",
            "</script>"
    })
    List<MobileStatisticsReturnVO> visualVisitSort(List<String> types, List<Long> ids, Integer limit);




}
