/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.visual.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.visual.dto.ComponentLineChartDto;
import org.springblade.visual.dto.ComponentTopDto;
import org.springblade.visual.entity.VisualComponent;

import java.util.List;

/**
 * 报表模板 Mapper 接口
 *
 * @author BladeX
 * @since 2021-07-02
 */
public interface VisualComponentMapper extends BaseMapper<VisualComponent> {



    /**
     * 根据日期分组 查询组件的使用总次数和人数
     *
     * @return
     */
    @Select("<script>" +
            //"set session tidb_opt_distinct_agg_push_down = 1;" +
            "SELECT DATE_FORMAT(create_time, '%Y-%m-%d') AS date, COUNT(*) AS count, COUNT(DISTINCT create_user_id) AS userCount " +
            "FROM visual_component " +
            "WHERE 1=1 " +
            "<if test='componentType == null'> AND component_source != 3 </if>" +
            "<if test='componentType != null'> AND component_source = #{componentType} </if>" +
            "<if test='componentType == null'> AND component_source is not null </if>" +
            "<if test='tenantId != null'> AND tenant_id = #{tenantId} </if>" +
            "<if test='startDate != null and endDate != null'> AND create_time BETWEEN #{startDate} AND #{endDate} </if>" +
            "GROUP BY date order by date" +
            "</script>")
    List<ComponentLineChartDto> lineChart(@Param("componentType") Integer componentType,
                                          @Param("tenantId") String tenantId,
                                          @Param("componentName") String componentName,
                                          @Param("startDate") String startDate,
                                          @Param("endDate") String endDate);

    @Select("SELECT component_source_id as id,count(DISTINCT create_user_id) as userCount, COUNT(*) AS count FROM visual_component WHERE component_source = #{componentSource} GROUP BY component_source_id ORDER BY count DESC LIMIT #{top};")
    List<ComponentTopDto> componentTop(@Param("componentSource") Integer componentSource, @Param("top") Integer top);


    @Select("<script>" +
            "SELECT vc.*,vctc.`name` componentTemplateCategory FROM visual_component vc left join visual_component_template vct on vc.component_source_id = vct.id LEFT JOIN visual_component_template_category vctc ON vctc.id = vct.mcp_category WHERE vc.component_source = 3  " +
            "<if test='tenantId != null'> AND vc.tenant_id = #{tenantId} </if>" +
            "<if test='categoryId != null'> AND vctc.id = #{categoryId} </if>" +
            "<if test='componentName != null'> AND vc.title like CONCAT('%', #{componentName}, '%') </if>" +
            "<if test='startDate != null and endDate != null'> AND vc.create_time BETWEEN #{startDate} AND #{endDate} </if>" +
            " order by vc.create_time desc" +
            "</script>")
    Page<VisualComponent> componentTemplatePage(@Param("categoryId") Long categoryId,
                                                @Param("tenantId") String tenantId,
                                                @Param("componentName") String componentName,
                                                @Param("startDate") String startDate,
                                                @Param("endDate") String endDate,
                                                @Param("page") Page page);

    @Select("<script>" +
            "SELECT vc.*,vctc.`name` componentTemplateCategory FROM visual_component vc left join visual_component_template vct on vc.component_source_id = vct.id LEFT JOIN visual_component_template_category vctc ON vctc.id = vct.mcp_category WHERE vc.component_source = 3  " +
            "<if test='tenantId != null'> AND vc.tenant_id = #{tenantId} </if>" +
            "<if test='categoryId != null'> AND vctc.id = #{categoryId} </if>" +
            "<if test='componentName != null'> AND vct.name like CONCAT('%', #{componentName}, '%') </if>" +
            "<if test='startDate != null and endDate != null'> AND vc.create_time BETWEEN #{startDate} AND #{endDate} </if>" +
            " order by vc.create_time desc" +
            "</script>")
    List<VisualComponent> componentTemplateList(@Param("categoryId") Long categoryId,
                                                @Param("tenantId") String tenantId,
                                                @Param("componentName") String componentName,
                                                @Param("startDate") String startDate,
                                                @Param("endDate") String endDate);

    /**
     * 根据日期分组 查询组件的使用总次数和人数
     *
     * @return
     */
    @Select("<script>" +
            "SELECT DATE_FORMAT(vc.create_time, '%Y-%m-%d') AS date, COUNT(1) AS count, COUNT(DISTINCT vc.create_user_id) AS userCount FROM visual_component vc left join visual_component_template vct on vc.component_source_id = vct.id LEFT JOIN visual_component_template_category vctc ON vctc.id = vct.mcp_category WHERE vc.component_source = 3  " +
            "<if test='tenantId != null'> AND vc.tenant_id = #{tenantId} </if>" +
            "<if test='categoryId != null'> AND vctc.id = #{categoryId} </if>" +
            "<if test='componentName != null'> AND vct.name like CONCAT('%', #{componentName}, '%') </if>" +
            "<if test='startDate != null and endDate != null'> AND DATE(vc.create_time) BETWEEN #{startDate} AND #{endDate} </if>" +
            "GROUP BY date order by date" +
            "</script>")
    List<ComponentLineChartDto> templateLineChart(@Param("categoryId") Long categoryId,
                                                  @Param("tenantId") String tenantId,
                                                  @Param("componentName") String componentName,
                                                  @Param("startDate") String startDate,
                                                  @Param("endDate") String endDate);


}
