package org.springblade.visual.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.utils.Base62Util;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.MobileStatisticsQueryByIdParams;
import org.springblade.visual.dto.MobileStatisticsQueryParams;
import org.springblade.visual.dto.MobileStatisticsReturnVO;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataShare;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springblade.visual.enums.MobileStatisticsSourceEnum;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.CkMapper;
import org.springblade.visual.mapper.VisualDataTemplateMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author xipengqiang
 */
@Service
public class MobileStatisticsService {

    @Autowired
    private VisualDataTemplateMapper visualDataTemplateMapper;

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private VisualDataShareService visualDataShareService;

    @Autowired
    private CkMapper ckMapper;

    @Autowired
    private DataPermClient dataPermClient;

    @Value("${sys.domain:http://bi.9n1m.com/}")
    private String domain;

    /**
     * 模板库数据
     */
    public List<MobileStatisticsReturnVO> template(MobileStatisticsQueryParams params) {
        //获取所有移动端模板
        List<VisualDataTemplate> visualDataTemplates = visualDataTemplateMapper.allMobileReviewedTemplate();
        //获取访问量排序查询
        if (visualDataTemplates.size() != 0) {
            Map<Long, VisualDataTemplate> templateMap = visualDataTemplates.stream().collect(Collectors.toMap(VisualDataTemplate::getVisualId, Function.identity()));
            List<Long> ids = visualDataTemplates.stream().map(VisualDataTemplate::getVisualId).collect(Collectors.toList());
            List<MobileStatisticsReturnVO> returnVOList = ckMapper.visualVisitSort(Arrays.asList("模板"), ids, params.getLimit() > 500 ? 500 : params.getLimit());
            for (MobileStatisticsReturnVO returnVO : returnVOList) {
                VisualDataTemplate visualDataTemplate = templateMap.get(returnVO.getVisualId());
                if (visualDataTemplate != null) {
                    returnVO
                            .setVisualDesc(visualDataTemplate.getTemplateDesc())
                            .setVisualUrl(domain+"#/newbi/template/preview/"+returnVO.getVisualId())
                            .setAppSource(MobileStatisticsSourceEnum.TEMPLATE)
                            .setCreateTime(DateUtil.formatDateTime(visualDataTemplate.getCreateTime()))
                            .setVisualPic(visualDataTemplate.getPreviewImgUrl());
                }
            }
            return returnVOList;
        }
        return Collections.emptyList();
    }

    /**
     * 大屏数据
     */
    public List<MobileStatisticsReturnVO> visual(MobileStatisticsQueryParams params) {
        String tenantId = Base62Util.encode(params.getInstId());
        Long userId = params.getUserId();
        //查询可以看到的大屏
        LambdaQueryChainWrapper<VisualData> queryWrapper = visualDataService.lambdaQuery()
                .eq(VisualData::getVisualType, VisualTypeEnum.GENERAL)
                .eq(VisualData::getIsMobile, 1)
                .eq(TenantEntity::getTenantId, tenantId);
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
        if (checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.and(t -> t.eq(VisualData::getCreateUser, userId)
                    .or().inSql(VisualData::getId, "select a.self_report_id from visual_self_report_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
                    .or().eq(VisualData::getPublicStatus, 1));
        }
        List<VisualData> visualDataList = queryWrapper.list();
        //查询分享的大屏
        List<VisualDataShare> shareList = visualDataShareService.lambdaQuery()
                .eq(VisualDataShare::getSharedUser, userId)
                .eq(VisualDataShare::getSharedUserTenant, tenantId)
                .list();
        //组合大屏id
        List<Long> ids = new ArrayList<>();
        Map<Long, VisualData> visualDataMap = new HashMap<>();
        Map<Long, VisualDataShare> shareMap = new HashMap<>();
        if (shareList.size() != 0) {
            shareMap = shareList.stream().collect(Collectors.toMap(VisualDataShare::getRelationId, Function.identity()));
            ids.addAll(shareMap.keySet());
            //查询分享的大屏信息
            List<VisualData> visualData = visualDataService.getBaseMapper().listByIds(shareMap.keySet().stream().toList());
            visualDataList.addAll(visualData);
            visualDataList = visualDataList.stream().distinct().collect(Collectors.toList());
        }
        if (visualDataList.size() != 0) {
            visualDataMap = visualDataList.stream().collect(Collectors.toMap(VisualData::getId, Function.identity()));
            ids.addAll(visualDataMap.keySet());
        }
        //查询排序
        if (ids.size() != 0) {
            List<MobileStatisticsReturnVO> returnVOList = ckMapper.visualVisitSort(Arrays.asList("可视化大屏", "自助报表"), ids, params.getLimit() > 500 ? 500 : params.getLimit());
            for (MobileStatisticsReturnVO returnVO : returnVOList) {
                VisualData visualData = visualDataMap.get(returnVO.getVisualId());
                if (visualData != null) {
                    returnVO
                            .setVisualDesc(visualData.getVisualDesc())
                            .setVisualUrl(domain+"#/newbi/fullscreen/"+returnVO.getVisualId())
                            .setCreateTime(LocalDateTimeUtil.formatNormal(visualData.getCreateTime()))
                            .setVisualPic(visualData.getBackgroundUrl());
                    //判断数据来源
                    VisualDataShare visualDataShare = shareMap.get(visualData.getId());
                    if (visualDataShare != null) {
                        returnVO.setAppSource(MobileStatisticsSourceEnum.SHARE)
                                .setSharerUserId(visualDataShare.getCreateUser());
                    } else if (visualData.getCreateUser().equals(userId)) {
                        returnVO.setAppSource(MobileStatisticsSourceEnum.CREATE);
                    } else {
                        returnVO.setAppSource(MobileStatisticsSourceEnum.COMPANY);
                    }
                }
            }
            return returnVOList;
        }
        return Collections.emptyList();

    }

    /**
     * 通过应用id查询
     */
    public List<MobileStatisticsReturnVO> queryByAppId(MobileStatisticsQueryByIdParams params) {
        String tenantId = Base62Util.encode(params.getInstId());
        if (CollectionUtil.isNotEmpty(params.getAppList())) {
            Map<Long, MobileStatisticsSourceEnum> sourceEnumMap = params.getAppList().stream().collect(Collectors.toMap(MobileStatisticsQueryByIdParams.App::getId, MobileStatisticsQueryByIdParams.App::getAppSource));
            //模板
            List<MobileStatisticsQueryByIdParams.App> templates = params.getAppList().stream().filter(app -> app.getAppSource().equals(MobileStatisticsSourceEnum.TEMPLATE)).collect(Collectors.toList());
            List<MobileStatisticsQueryByIdParams.App> visuals = params.getAppList().stream().filter(app -> !app.getAppSource().equals(MobileStatisticsSourceEnum.TEMPLATE)).collect(Collectors.toList());
            List<MobileStatisticsReturnVO> returnVOs = new ArrayList<>();
            if (templates.size() != 0) {
                List<MobileStatisticsReturnVO> returnVOList = ckMapper.visualVisitSort(Arrays.asList("模板"), templates.stream().map(MobileStatisticsQueryByIdParams.App::getId).toList(), null);
                returnVOs.addAll(returnVOList);
            }
            if (visuals.size() != 0) {
                List<MobileStatisticsReturnVO> returnVOList = ckMapper.visualVisitSort(Arrays.asList("可视化大屏", "自助报表"), visuals.stream().map(MobileStatisticsQueryByIdParams.App::getId).toList(), null);
                returnVOs.addAll(returnVOList);
            }
            if (returnVOs.size() != 0) {
                //查找大屏信息
                List<Long> visualIds = returnVOs.stream().map(MobileStatisticsReturnVO::getVisualId).collect(Collectors.toList());
                List<VisualData> visualDataList = visualDataService.getBaseMapper().listByIds(visualIds);
                Map<Long, VisualData> visualDataMap = visualDataList.stream().collect(Collectors.toMap(VisualData::getId, Function.identity()));
                for (MobileStatisticsReturnVO returnVO : returnVOs) {
                    MobileStatisticsSourceEnum mobileStatisticsSourceEnum = sourceEnumMap.get(returnVO.getVisualId());
                    VisualData visualData = visualDataMap.get(returnVO.getVisualId());
                    if (visualData != null) {
                        returnVO
                                .setVisualDesc(visualData.getVisualDesc())
                                .setVisualUrl(domain+"#/newbi/fullscreen/"+returnVO.getVisualId())
                                .setCreateTime(LocalDateTimeUtil.formatNormal(visualData.getCreateTime()))
                                .setVisualPic(visualData.getBackgroundUrl());
                        switch (mobileStatisticsSourceEnum) {
                            case SHARE -> {
                                returnVO.setAppSource(MobileStatisticsSourceEnum.SHARE);
                                List<VisualDataShare> shareList = visualDataShareService.lambdaQuery()
                                        .eq(VisualDataShare::getSharedUser, params.getUserId())
                                        .eq(VisualDataShare::getSharedUserTenant, tenantId)
                                        .list();
                                if (shareList.size() != 0) {
                                    returnVO.setSharerUserId(shareList.get(0).getCreateUser());
                                }
                            }
                            case TEMPLATE -> {
                                returnVO
                                        .setVisualUrl(domain+"#/newbi/template/preview/"+returnVO.getVisualId())
                                        .setAppSource(MobileStatisticsSourceEnum.TEMPLATE);
                                VisualDataTemplate visualDataTemplate = visualDataTemplateMapper.getByVisualDataId(returnVO.getVisualId());
                                if (visualDataTemplate != null) {
                                    returnVO
                                            .setVisualDesc(visualDataTemplate.getTemplateDesc())
                                            .setVisualPic(visualDataTemplate.getPreviewImgUrl());
                                }
                            }
                            case CREATE -> {
                                returnVO.setAppSource(MobileStatisticsSourceEnum.CREATE);
                            }
                            case COMPANY -> {
                                returnVO.setAppSource(MobileStatisticsSourceEnum.COMPANY);
                            }
                        }
                    }
                }
                return returnVOs;
            }

        }
        return Collections.emptyList();
    }

}
