package org.springblade.visual.service;

import cn.hutool.core.codec.Base62;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.utils.Base62Util;
import org.springblade.visual.dto.MyVisualDataShareParams;
import org.springblade.visual.dto.VisualDataSharePageParams;
import org.springblade.visual.dto.VisualDataShareParams;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataShare;
import org.springblade.visual.mapper.VisualDataShareMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.util.DecodeUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class VisualDataShareService extends ServiceImpl<VisualDataShareMapper, VisualDataShare> {

    @Autowired
    private VisualDataService visualDataService;

    /**
     * 分享
     *
     * @param params
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean share(VisualDataShareParams params) {
        //先删除之前的分享  然后重新添加
        this.remove(this.lambdaUpdate().eq(VisualDataShare::getRelationId, params.getRelationId()).eq(VisualDataShare::getType, params.getType()).getWrapper());
        String relationName = "";
        if (params.getType() == 0 || params.getType() == 1) {
            VisualData visualData = visualDataService.getById(params.getRelationId());
            relationName = visualData.getTitle();
        }
        ArrayList<VisualDataShare> visualDataShares = new ArrayList<>();
        for (VisualDataShareParams.SharedUser sharedUser : params.getSharedUsers()) {
            VisualDataShare visualDataShare = new VisualDataShare();
            visualDataShare
                    .setCreateUser(AuthUtil.getUserId())
                    .setCreateUserName(AuthUtil.getNickName())
                    .setCreateTime(LocalDateTime.now())
                    .setRelationId(params.getRelationId())
                    .setRelationName(relationName)
                    .setType(params.getType());
            visualDataShare
                    .setSharedUser(sharedUser.getUserId())
                    .setSharedUserName(sharedUser.getNickName())
                    .setSharedUserTenant(sharedUser.getTenantId());

            visualDataShares.add(visualDataShare);
        }
        return this.saveBatch(visualDataShares);
    }

    public Page<VisualDataShare> page(VisualDataSharePageParams params) {
        Page<VisualDataShare> page = this.lambdaQuery()
                .eq(VisualDataShare::getSharedUser, AuthUtil.getUserId())
                .eq(VisualDataShare::getSharedUserTenant, AuthUtil.getTenantId())
                .eq(VisualDataShare::getType, params.getType())
                .like(StrUtil.isNotBlank(params.getName()), VisualDataShare::getRelationName, params.getName())
                .orderByDesc(VisualDataShare::getCreateTime)
                .page(new Page<>(params.getPage(), params.getSize()));
        if (page.getRecords().size() != 0) {
            List<Long> list = page.getRecords().stream().map(VisualDataShare::getRelationId).collect(Collectors.toList());
            List<VisualData> visualData = visualDataService.listByIds(list);
            Map<Long, VisualData> map = visualData.stream().collect(Collectors.toMap(VisualData::getId, Function.identity()));
            for (VisualDataShare record : page.getRecords()) {
                record.setVisualData(map.get(record.getRelationId()));
            }
        }
        return page;
    }

    public List<VisualDataShare> myShare(MyVisualDataShareParams params) {
        return this.lambdaQuery()
                .eq(VisualDataShare::getCreateUser, AuthUtil.getUserId())
                .eq(VisualDataShare::getRelationId, params.getRelationId())
                .eq(VisualDataShare::getType, params.getType())
                .list();
    }

    public VisualDataShare getOne(Long sharedUserId, Long relationId, List types) {
        VisualDataShare dataShare = this.lambdaQuery()
                .eq(VisualDataShare::getSharedUser, sharedUserId)
                .eq(VisualDataShare::getRelationId, relationId)
                .eq(VisualDataShare::getSharedUserTenant,AuthUtil.getTenantId())
                .in(VisualDataShare::getType, types)
                .one();
        return dataShare;
    }

}

